@@ -26,6 +26,7 @@
 
 #include <osmocom/core/crc8gen.h>
 #include <osmocom/codec/codec.h>
+#include <osmocom/gsm/rtp_extensions.h>
 
 #include <osmocom/trau/trau_frame.h>
 #include <osmocom/trau/trau_rtp.h>
@@ -47,7 +48,7 @@ static const uint8_t gsm_fr_map[] = {
 
 
 /*
- * EFR TRAU parity
+ * EFR TRAU parity (also used for HR)
  *
  * g(x) = x^3 + x^1 + 1
  */
@@ -100,20 +101,34 @@ static inline void efr_parity_bits_5(ubit_t *check_bits, const ubit_t *d_bits)
  *  \param[in] out_len length of out buffer in bytes
  *  \param[in] fr input TRAU frame in decoded form
  *  \returns number of bytes generated in 'out'; negative on error. */
-static int trau2rtp_fr(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf)
+static int trau2rtp_fr(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf, bool emit_twts001)
 {
+	size_t req_out_len = emit_twts001 ? GSM_FR_BYTES+1 : GSM_FR_BYTES;
 	int i, j, k, l, o;
 
 	if (tf->type != OSMO_TRAU16_FT_FR)
 		return -EINVAL;
 
+	if (out_len < req_out_len)
+		return -ENOSPC;
+
 	/* FR Data Bits according to TS 48.060 Section 5.5.1.1.2 */
 
-	if (tf->c_bits[11]) /* BFI */
-		return 0;
+	/* Are we emitting TW-TS-001? If so, emit TRAU-like Extension Header */
+	if (emit_twts001) {
+		uint8_t hdr_byte = 0xE0;
 
-	if (out_len < GSM_FR_BYTES)
-		return -ENOSPC;
+		if (tf->c_bits[16])	/* DTXd */
+			hdr_byte |= 0x08;
+		if (tf->c_bits[11])	/* BFI */
+			hdr_byte |= 0x02;
+		if (tf->c_bits[14])	/* TAF */
+			hdr_byte |= 0x01;
+		*out++ = hdr_byte;
+	}
+
+	if (tf->c_bits[11] && !emit_twts001) /* BFI without TW-TS-001 */
+		return 0;
 
 	out[0] = 0xd << 4;
 	/* reassemble d-bits */
@@ -135,55 +150,207 @@ static int trau2rtp_fr(uint8_t *out, size_t out_len, const struct osmo_trau_fram
 		j++;
 	}
 
-	return GSM_FR_BYTES;
+	return req_out_len;
 }
 
-/* See Section 5.2 of RFC5993 */
-enum rtp_hr_ietf_ft {
-	FT_GOOD_SPEECH  = 0,
-	FT_GOOD_SID     = 2,
-	FT_NO_DATA      = 7,
+/* See Section 5.2 of RFC5993 and TW-TS-002 */
+enum super5993_ft {
+	FT_GOOD_SPEECH		= 0,
+	FT_INVALID_SID		= 1,
+	FT_GOOD_SID		= 2,
+	FT_BFI_WITH_DATA	= 6,
+	FT_NO_DATA		= 7,
 };
 
-static const uint8_t rtp_hr_sid[14] = { 0x00, 0x00, 0x00, 0x00, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
+static void twts002_hr16_set_extra_flags(uint8_t *out, const struct osmo_trau_frame *tf)
+{
+	if (tf->c_bits[16])	/* DTXd */
+		out[0] |= 0x08;
+	if (tf->ufi)		/* UFI */
+		out[0] |= 0x02;
+	if (tf->c_bits[14])	/* TAF */
+		out[0] |= 0x01;
+}
 
-/*! Generate the 14 bytes ETSI TS 101 318 RTP payload for HR from a decoded 16k TRAU frame.
- *  Note that thsi differs from the IETF RFC5993 format.  However, as OsmoBTS implements
- *  the TS 101 318 format, we also use the same format here. osmo-mgw can convert them.
+/*! Generate an RFC 5993 RTP payload for HR from a decoded 16k TRAU frame.
+ *  We previously emitted TS 101 318 format; however, RFC 5993 is the format
+ *  specified in TS 48.103 for AoIP, it can also be extended with TRAU-UL-like
+ *  capabilities with TW-TS-002, and we now support RFC 5993 output in OsmoBTS
+ *  on all models.
  *  \param[out] out caller-provided output buffer
  *  \param[in] out_len length of out buffer in bytes
  *  \param[in] tf input TRAU frame in decoded form
  *  \returns number of bytes generated in 'out'; negative on error. */
-static int trau2rtp_hr16(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf)
+static int trau2rtp_hr16(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf, bool emit_twts002)
 {
-	unsigned int i;
+	enum osmo_gsm631_sid_class sidc;
+	int rc;
 
 	if (tf->type != OSMO_TRAU16_FT_HR)
 		return -EINVAL;
 
+	if (out_len < GSM_HR_BYTES_RTP_RFC5993)
+		return -ENOSPC;
+
 	/* HR Data Bits according to TS 48.061 Section 5.1.4.1.1 */
 
-	if (tf->dir == OSMO_TRAU_DIR_UL && tf->c_bits[11]) /* C12: BFI */
+	/* bad CRC means bad frame, no matter what else is going on */
+	rc = osmo_crc8gen_check_bits(&gsm0860_efr_crc3, tf->d_bits, 44,
+				     tf->crc_bits);
+	if (rc)
 		goto bad_frame;
 
-	if (out_len < GSM_HR_BYTES)
-		return -ENOSPC;
+	sidc = (tf->c_bits[12] << 1) | tf->c_bits[13];
+	/* both C13 and C14 being set is invalid combination */
+	if (sidc > OSMO_GSM631_SID_CLASS_VALID)
+		goto bad_frame;
+
+	/* Plain RFC 5993 without TW-TS-002 extensions does not allow
+	 * BFI or invalid SID packets. */
+	if (!emit_twts002 &&
+	    (tf->c_bits[11] || sidc == OSMO_GSM631_SID_CLASS_INVALID))
+		goto bad_frame;
+
+	/* BFI turns valid SID into invalid */
+	if (tf->c_bits[11] && sidc == OSMO_GSM631_SID_CLASS_VALID)
+		sidc = OSMO_GSM631_SID_CLASS_INVALID;
+
+	/* RFC 5993 Frame Type is equal to GSM 06.41 SID classification,
+	 * restricted to just speech or valid SID per above.  Or if we
+	 * emit TW-TS-002, that restriction is lifted. */
+	out[0] = sidc << 4;
+
+	if (emit_twts002) {
+		if (tf->c_bits[11] && sidc == OSMO_GSM631_SID_CLASS_SPEECH)
+			out[0] = FT_BFI_WITH_DATA << 4;
+		twts002_hr16_set_extra_flags(out, tf);
+		/* invalid SID frames are truncated in TW-TS-002 */
+		if (sidc == OSMO_GSM631_SID_CLASS_INVALID)
+			return 1;
+	}
 
 	/* TS 101 318 Section 5.2: The order of occurrence of the codec parameters in the buffer is
 	 * the same as order of occurrence over the Abis as defined in annex B of ETS 300 969
 	 * [which is 3GPP TS 46.020 */
-	osmo_ubit2pbit(out, tf->d_bits, 112);
+	osmo_ubit2pbit(out + 1, tf->d_bits, 112);
 
-	if (tf->c_bits[12] || tf->c_bits[13]) {
-		/* Generate SID frame as per TS 101 318 Section 5.2.2 */
-		for (i = 0; i < sizeof(rtp_hr_sid); i++)
-			out[i] = out[i] | rtp_hr_sid[i];
-	}
+	/* RFC 5993 requires SID frames to be perfect, error-free */
+	if (!emit_twts002 && sidc == OSMO_GSM631_SID_CLASS_VALID)
+		osmo_hr_sid_reset(out + 1);
 
-	return GSM_HR_BYTES;
+	return GSM_HR_BYTES_RTP_RFC5993;
 
 bad_frame:
-	return 0;
+	if (emit_twts002) {
+		out[0] = FT_NO_DATA << 4;
+		twts002_hr16_set_extra_flags(out, tf);
+		return 1;
+	} else
+		return 0;
+}
+
+static bool hr8_xc_bits_good_parity(const ubit_t *xc_bits)
+{
+	int i;
+	unsigned int sum = 0;
+
+	for (i = 0; i < 6; i++)
+		sum += xc_bits[i];
+	if (sum & 1)
+		return true;	/* odd sum good */
+	else
+		return false;	/* even sum bad */
+}
+
+static void twts002_hr8_set_extra_flags(uint8_t *out, const struct osmo_trau_frame *tf)
+{
+	if (tf->c_bits[8])	/* DTXd */
+		out[0] |= 0x08;
+	if (tf->xc_bits[4])	/* UFI */
+		out[0] |= 0x02;
+}
+
+/*! Generate an RFC 5993 or TW-TS-002 RTP payload for HRv1
+ *  from a decoded 8k TRAU frame.
+ *  \param[out] out caller-provided output buffer
+ *  \param[in] out_len length of out buffer in bytes
+ *  \param[in] tf input TRAU frame in decoded form
+ *  \param[in] emit_twts002 self-explanatory
+ *  \returns number of bytes generated in 'out'; negative on error. */
+static int trau2rtp_hr8(uint8_t *out, size_t out_len,
+			const struct osmo_trau_frame *tf, bool emit_twts002)
+{
+	unsigned xc1_4;
+
+	/* function interface preliminaries */
+	if (tf->type != OSMO_TRAU8_SPEECH)
+		return -EINVAL;
+	if (out_len < GSM_HR_BYTES_RTP_RFC5993)
+		return -ENOSPC;
+
+	/* Before considering anything else, if we have bad parity in
+	 * frame classification bits or bad CRC in payload bits,
+	 * we treat this frame as totally invalid: BFI with no data. */
+	if (!hr8_xc_bits_good_parity(tf->xc_bits) ||
+	    osmo_crc8gen_check_bits(&gsm0860_efr_crc3, tf->d_bits, 44,
+				    tf->crc_bits))
+		goto bad_frame;
+
+	/* classify this frame per XC1..XC4 */
+	xc1_4 = (tf->xc_bits[0] << 3) | (tf->xc_bits[1] << 2) |
+		(tf->xc_bits[2] << 1) | (tf->xc_bits[3] << 0);
+	switch (xc1_4) {
+	case 0:
+		/* good speech frame (BFI=0, SID=0) */
+		out[0] = FT_GOOD_SPEECH << 4;
+		if (emit_twts002)
+			twts002_hr8_set_extra_flags(out, tf);
+		osmo_ubit2pbit(out + 1, tf->d_bits, 112);
+		return GSM_HR_BYTES_RTP_RFC5993;
+	case 1:
+		/* valid SID frame (BFI=0, SID=2) */
+		out[0] = FT_GOOD_SID << 4;
+		if (emit_twts002)
+			twts002_hr8_set_extra_flags(out, tf);
+		osmo_ubit2pbit(out + 1, tf->d_bits, 112);
+		/* RFC 5993 requires SID frames to be perfect, error-free */
+		if (!emit_twts002)
+			osmo_hr_sid_reset(out + 1);
+		return GSM_HR_BYTES_RTP_RFC5993;
+	case 4:
+	case 5:
+		/* invalid SID frame (multiple BFI/SID combinations) */
+		/* can be represented only in TW-TS-002, not in RFC 5993 */
+		if (!emit_twts002)
+			return 0;
+		out[0] = FT_INVALID_SID << 4;
+		twts002_hr8_set_extra_flags(out, tf);
+		/* XC4 is TAF with this frame type */
+		if (tf->xc_bits[3])
+			out[0] |= 0x01;
+		return 1;	/* short format per TW-TS-002 */
+	case 6:
+	case 7:
+		/* bad speech frame (BFI=1, SID=0) */
+		if (!emit_twts002)
+			return 0;
+		out[0] = FT_BFI_WITH_DATA << 4;
+		twts002_hr8_set_extra_flags(out, tf);
+		/* XC4 is TAF with this frame type */
+		if (tf->xc_bits[3])
+			out[0] |= 0x01;
+		osmo_ubit2pbit(out + 1, tf->d_bits, 112);
+		return GSM_HR_BYTES_RTP_RFC5993;
+	default:
+bad_frame:
+		/* received garbage, emit BFI with no data */
+		if (emit_twts002) {
+			out[0] = FT_NO_DATA << 4;
+			twts002_hr8_set_extra_flags(out, tf);
+			return 1;
+		} else
+			return 0;
+	}
 }
 
 /*! Generate the 31 bytes RTP payload for GSM-EFR from a decoded TRAU frame.
@@ -191,25 +358,36 @@ bad_frame:
  *  \param[in] out_len length of out buffer in bytes
  *  \param[in] fr input TRAU frame in decoded form
  *  \returns number of bytes generated in 'out'; negative on error. */
-static int trau2rtp_efr(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf)
+static int trau2rtp_efr(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf, bool emit_twts001)
 {
+	size_t req_out_len = emit_twts001 ? GSM_EFR_BYTES+1 : GSM_EFR_BYTES;
 	int i, j, rc;
 	ubit_t check_bits[26];
+	uint8_t *twts001_hdr;
+
+	if (out_len < req_out_len)
+		return -ENOSPC;
 
 	if (tf->type != OSMO_TRAU16_FT_EFR)
 		return -EINVAL;
 
 	/* FR Data Bits according to TS 48.060 Section 5.5.1.1.2 */
 
-	if (tf->c_bits[11]) /* BFI */
+	/* Are we emitting TW-TS-001? If so, emit TRAU-like Extension Header */
+	if (emit_twts001) {
+		twts001_hdr = out++;
+		*twts001_hdr = 0xE0;
+		if (tf->c_bits[16])	/* DTXd */
+			*twts001_hdr |= 0x08;
+		if (tf->c_bits[11])	/* BFI */
+			*twts001_hdr |= 0x02;
+		if (tf->c_bits[14])	/* TAF */
+			*twts001_hdr |= 0x01;
+	}
+
+	if (tf->c_bits[11] && !emit_twts001) /* BFI without TW-TS-001 */
 		return 0;
 
-	if (out_len < GSM_EFR_BYTES)
-		return -ENOSPC;
-
-	if (tf->c_bits[11]) /* BFI */
-		goto bad_frame;
-
 	out[0] = 0xc << 4;
 	/* reassemble d-bits */
 	for (i = 1, j = 4; i < 39; i++, j++)
@@ -248,9 +426,14 @@ static int trau2rtp_efr(uint8_t *out, size_t out_len, const struct osmo_trau_fra
 	if (rc)
 		goto bad_frame;
 
-	return GSM_EFR_BYTES;
+	return req_out_len;
+
 bad_frame:
-	return 0;
+	if (emit_twts001) {
+		*twts001_hdr |= 0x06;	/* BFI and No_Data flags */
+		return 1;
+	} else
+		return 0;
 }
 
 /* TS 48.060 Section 5.5.1.1.2 */
@@ -258,13 +441,29 @@ static int rtp2trau_fr(struct osmo_trau_frame *tf, const uint8_t *data, size_t d
 {
 	int i, j, k, l, o;
 	enum osmo_gsm631_sid_class sidc;
+	bool bfi = false, taf = false, dtxd = false;
 
-	/* data_len == 0 for BFI frame */
-	if (data_len < GSM_FR_BYTES && data_len != 0)
-		return -EINVAL;
+	/* accept TW-TS-001 input */
+	if (data_len > 0 && (data[0] & 0xF0) == 0xE0) {
+		dtxd = (data[0] & 0x08) != 0;
+		bfi  = (data[0] & 0x02) != 0;
+		taf  = (data[0] & 0x01) != 0;
+		data++;
+		data_len--;
+	}
 
-	if (data_len && data[0] >> 4 != 0xd)
+	/* now we must have either standard FR or no-data input */
+	switch (data_len) {
+	case GSM_FR_BYTES:
+		if ((data[0] & 0xF0) != 0xD0)
+			return -EINVAL;
+		break;
+	case 0:
+		bfi = true;
+		break;
+	default:
 		return -EINVAL;
+	}
 
 	tf->type = OSMO_TRAU16_FT_FR;
 
@@ -297,20 +496,19 @@ static int rtp2trau_fr(struct osmo_trau_frame *tf, const uint8_t *data, size_t d
 	}
 	memset(&tf->c_bits[5], 0, 6);	/* C6 .. C11: Time Alignment */
 	if (tf->dir == OSMO_TRAU_DIR_UL) {
+		tf->c_bits[11] = bfi;		/* C12: BFI */
 		if (data_len == 0) {
-			tf->c_bits[11] = 1;	/* C12: BFI */
 			tf->c_bits[12] = 0;	/* C13: SID=0 */
 			tf->c_bits[13] = 0;	/* C14: SID=0 */
 		} else {
-			tf->c_bits[11] = 0;	/* C12: BFI */
 			/* SID classification per GSM 06.31 section 6.1.1 */
 			sidc = osmo_fr_sid_classify(data);
 			tf->c_bits[12] = (sidc >> 1) & 1; /* C13: msb */
 			tf->c_bits[13] = (sidc >> 0) & 1; /* C14: lsb */
 		}
-		tf->c_bits[14] = 0; /* C15: TAF (SACCH or not) */
-		tf->c_bits[15] = 1; /* C16: spare */
-		tf->c_bits[16] = 0; /* C17: DTXd not applied */
+		tf->c_bits[14] = taf;  /* C15: TAF (SACCH or not) */
+		tf->c_bits[15] = 1;    /* C16: spare */
+		tf->c_bits[16] = dtxd; /* C17: DTXd applied or not */
 	} else {
 		memset(&tf->c_bits[11], 1, 4);	/* C12 .. C15: spare */
 		if (data_len && osmo_fr_check_sid(data, data_len))
@@ -350,25 +548,22 @@ static int rtp2trau_fr(struct osmo_trau_frame *tf, const uint8_t *data, size_t d
 	return 0;
 }
 
-/* does the RTP HR payload resemble a SID frame or not */
-static bool is_rtp_hr_sid(const uint8_t *data, const uint8_t data_len)
-{
-	int i;
-
-	if (data_len < GSM_HR_BYTES)
-		return false;
-
-	for (i = 0; i < GSM_HR_BYTES; i++) {
-		if ((data[i] & rtp_hr_sid[i]) != rtp_hr_sid[i])
-			return false;
-	}
-	return true;
-}
-
 static int rtp2trau_hr16(struct osmo_trau_frame *tf, const uint8_t *data, size_t data_len)
 {
-	if (data_len < GSM_HR_BYTES && data_len != 0)
+	/* accept both TS 101 318 and RFC 5993 payloads */
+	switch (data_len) {
+	case GSM_HR_BYTES:
+		break;
+	case GSM_HR_BYTES_RTP_RFC5993:
+		data++;
+		data_len--;
+		break;
+	case 0:
+		/* accept no-data input */
+		break;
+	default:
 		return -EINVAL;
+	}
 
 	tf->type = OSMO_TRAU16_FT_HR;
 
@@ -395,7 +590,7 @@ static int rtp2trau_hr16(struct osmo_trau_frame *tf, const uint8_t *data, size_t
 			tf->c_bits[11] = 1;
 		else
 			tf->c_bits[11] = 0;
-		if (is_rtp_hr_sid(data, data_len)) {
+		if (osmo_hr_check_sid(data, data_len)) {
 			/* SID=2 is a valid SID frame */
 			tf->c_bits[12] = 1;
 			tf->c_bits[13] = 0;
@@ -407,11 +602,11 @@ static int rtp2trau_hr16(struct osmo_trau_frame *tf, const uint8_t *data, size_t
 		tf->c_bits[15] = 0; /* C16: SP */
 		tf->c_bits[16] = 0; /* C17: DTXd shall not be applied */
 	} else {
-		tf->c_bits[11] = 0; /* C12: UFE */
+		tf->c_bits[11] = 1; /* C12: UFE */
 		tf->c_bits[12] = 1; /* C13: spare */
 		tf->c_bits[13] = 1; /* C14: spare */
 		tf->c_bits[14] = 1; /* C15: spare */
-		if (is_rtp_hr_sid(data, data_len))
+		if (osmo_hr_check_sid(data, data_len))
 			tf->c_bits[15] = 0; /* C16: SP */
 		else
 			tf->c_bits[15] = 1; /* C16: SP */
@@ -423,9 +618,75 @@ static int rtp2trau_hr16(struct osmo_trau_frame *tf, const uint8_t *data, size_t
 		tf->ufi = 0;
 	else
 		tf->ufi = 1;
-	/* CRC is computed by TRAU frame encoder */
 	if (data_len)
-		memcpy(tf->d_bits, data, GSM_HR_BYTES);
+		osmo_pbit2ubit(tf->d_bits, data, GSM_HR_BYTES * 8);
+	else
+		memset(tf->d_bits, 0, GSM_HR_BYTES * 8);
+	/* CRC is *not* computed by TRAU frame encoder - we have to do it */
+	osmo_crc8gen_set_bits(&gsm0860_efr_crc3, tf->d_bits, 44, tf->crc_bits);
+
+	return 0;
+}
+
+static int rtp2trau_hr8(struct osmo_trau_frame *tf, const uint8_t *data, size_t data_len)
+{
+	/* accept both TS 101 318 and RFC 5993 payloads */
+	switch (data_len) {
+	case GSM_HR_BYTES:
+		break;
+	case GSM_HR_BYTES_RTP_RFC5993:
+		data++;
+		data_len--;
+		break;
+	case 0:
+		/* accept no-data input */
+		break;
+	default:
+		return -EINVAL;
+	}
+
+	/* FIXME: implement TRAU-UL frame generation if and when
+	 * someone actually needs it in a program that uses
+	 * this library. */
+	if (tf->dir != OSMO_TRAU_DIR_DL)
+		return -ENOTSUP;
+
+	tf->type = OSMO_TRAU8_SPEECH;
+
+	/* C1..C5 */
+	tf->c_bits[0] = 0;
+	tf->c_bits[1] = 0;
+	tf->c_bits[2] = 0;
+	tf->c_bits[3] = 1;	/* UFE: good uplink */
+	tf->c_bits[4] = 0;	/* odd parity */
+	/* C6..C9: spare bits */
+	memset(tf->c_bits + 5, 1, 4);
+
+	/* XC1..XC6 */
+	if (osmo_hr_check_sid(data, data_len)) {
+		tf->xc_bits[0] = 0;
+		tf->xc_bits[1] = 0;
+		tf->xc_bits[2] = 0;
+		tf->xc_bits[3] = 1;
+		tf->xc_bits[4] = 0;
+		tf->xc_bits[5] = 0;	/* odd parity */
+	} else {
+		tf->xc_bits[0] = 0;
+		tf->xc_bits[1] = 0;
+		tf->xc_bits[2] = 0;
+		tf->xc_bits[3] = 0;
+		tf->xc_bits[4] = 0;
+		tf->xc_bits[5] = 1;	/* odd parity */
+	}
+
+	memset(&tf->t_bits[0], 1, 2);
+
+	if (data_len)
+		osmo_pbit2ubit(tf->d_bits, data, GSM_HR_BYTES * 8);
+	else
+		memset(tf->d_bits, 0, GSM_HR_BYTES * 8);
+	/* CRC is *not* computed by TRAU frame encoder - we have to do it */
+	osmo_crc8gen_set_bits(&gsm0860_efr_crc3, tf->d_bits, 44, tf->crc_bits);
 
 	return 0;
 }
@@ -436,13 +697,29 @@ static int rtp2trau_efr(struct osmo_trau_frame *tf, const uint8_t *data, size_t
 	int i, j;
 	ubit_t check_bits[26];
 	enum osmo_gsm631_sid_class sidc;
+	bool bfi = false, taf = false, dtxd = false;
 
-	/* data_len == 0 for BFI frame */
-	if (data_len < GSM_EFR_BYTES && data_len != 0)
-		return -EINVAL;
+	/* accept TW-TS-001 input */
+	if (data_len > 0 && (data[0] & 0xF0) == 0xE0) {
+		dtxd = (data[0] & 0x08) != 0;
+		bfi  = (data[0] & 0x02) != 0;
+		taf  = (data[0] & 0x01) != 0;
+		data++;
+		data_len--;
+	}
 
-	if (data_len && data[0] >> 4 != 0xc)
+	/* now we must have either standard EFR or no-data input */
+	switch (data_len) {
+	case GSM_EFR_BYTES:
+		if ((data[0] & 0xF0) != 0xC0)
+			return -EINVAL;
+		break;
+	case 0:
+		bfi = true;
+		break;
+	default:
 		return -EINVAL;
+	}
 
 	tf->type = OSMO_TRAU16_FT_EFR;
 
@@ -467,20 +744,19 @@ static int rtp2trau_efr(struct osmo_trau_frame *tf, const uint8_t *data, size_t
 
 	memset(&tf->c_bits[5], 0, 6); /* C6 .. C11: Time Alignment */
 	if (tf->dir == OSMO_TRAU_DIR_UL) {
+		tf->c_bits[11] = bfi;		/* C12: BFI */
 		if (data_len == 0) {
-			tf->c_bits[11] = 1;	/* C12: BFI */
 			tf->c_bits[12] = 0;	/* C13: SID=0 */
 			tf->c_bits[13] = 0;	/* C14: SID=0 */
 		} else {
-			tf->c_bits[11] = 0;	/* C12: BFI */
 			/* SID classification per GSM 06.81 section 6.1.1 */
 			sidc = osmo_efr_sid_classify(data);
 			tf->c_bits[12] = (sidc >> 1) & 1; /* C13: msb */
 			tf->c_bits[13] = (sidc >> 0) & 1; /* C14: lsb */
 		}
-		tf->c_bits[14] = 0; /* C15: TAF (SACCH) */
-		tf->c_bits[15] = 1; /* C16: spare */
-		tf->c_bits[16] = 0; /* C17: DTXd applied */
+		tf->c_bits[14] = taf;  /* C15: TAF (SACCH or not) */
+		tf->c_bits[15] = 1;    /* C16: spare */
+		tf->c_bits[16] = dtxd; /* C17: DTXd applied or not */
 	} else {
 		tf->c_bits[11] = 1; /* C12: UFE (good uplink) */
 		memset(&tf->c_bits[12], 1, 3);	/* C13 .. C15: spare */
@@ -666,16 +942,34 @@ int trau2rtp_amr(uint8_t *out, const struct osmo_trau_frame *tf, enum osmo_amr_m
 }
 #endif
 
+static inline bool check_twts001(struct osmo_trau2rtp_state *st)
+{
+	if (st->rtp_extensions & OSMO_RTP_EXT_TWTS001)
+		return true;
+	else
+		return false;
+}
+
+static inline bool check_twts002(struct osmo_trau2rtp_state *st)
+{
+	if (st->rtp_extensions & OSMO_RTP_EXT_TWTS002)
+		return true;
+	else
+		return false;
+}
+
 int osmo_trau2rtp(uint8_t *out, size_t out_len, const struct osmo_trau_frame *tf,
 		  struct osmo_trau2rtp_state *st)
 {
 	switch (tf->type) {
 	case OSMO_TRAU16_FT_FR:
-		return trau2rtp_fr(out, out_len, tf);
+		return trau2rtp_fr(out, out_len, tf, check_twts001(st));
 	case OSMO_TRAU16_FT_EFR:
-		return trau2rtp_efr(out, out_len, tf);
+		return trau2rtp_efr(out, out_len, tf, check_twts001(st));
 	case OSMO_TRAU16_FT_HR:
-		return trau2rtp_hr16(out, out_len, tf);
+		return trau2rtp_hr16(out, out_len, tf, check_twts002(st));
+	case OSMO_TRAU8_SPEECH:
+		return trau2rtp_hr8(out, out_len, tf, check_twts002(st));
 	default:
 		return -EINVAL;
 	}
@@ -691,6 +985,8 @@ int osmo_rtp2trau(struct osmo_trau_frame *tf, const uint8_t *rtp, size_t rtp_len
 		return rtp2trau_efr(tf, rtp, rtp_len);
 	case OSMO_TRAU16_FT_HR:
 		return rtp2trau_hr16(tf, rtp, rtp_len);
+	case OSMO_TRAU8_SPEECH:
+		return rtp2trau_hr8(tf, rtp, rtp_len);
 	default:
 		return -EINVAL;
 	}
